//	mtm.h
//

#ifndef _MTM_H
#define _MTM_H

#define MAXTRK 32

/* Structures to hold the midi notes as we read them in. */
/* structure to hold basic note information */

typedef struct event
{
  struct event *next, *prev;
  char type;			/* see defines below */
  char tied;			/* set to 1 if part of tied sequence */
  long start, end, pitch, vel;
} event;

#define NOTE  1
#define PATCH 2
#define TEXT  3
#define TEMPO 4
#define PITCH 5
#define END   6
#define PARAM 7
#define BEND  8

/* main structure holding information on each track */

typedef struct track {
  event *notes;		/* list of all events for this track */
  event *p;		/* work pointer, next event to be processed */
  short minp;		/* lowest pitch in track */
  short maxp;		/* highest pitch in track */
  short clef;		/* basic clef */
  int notect;		/* number of notes (ignore other events) */
  int ct;		/* number of events in track */
} track;

extern track tracks[MAXTRK];	/* main event tracks */
extern track mtrks[MAXTRK];	/* secondary tracks for modifiers */
extern track *ct, *mct;		/* current track, and modifiers */

/*
 * Structure to hold time sigs as we read them
 * in.
 */
typedef struct {
  long when;
  short num;
  short dnom;
} Tsig;

extern Tsig tsig[100];

/* Keep track of how many new time sigs we have. */
extern short twhre;
/* measure number */
extern short measure;
/* Some stuff we use for getting the notes out. */
extern short tr[MAXTRK];
extern short vel[MAXTRK];		/* note of last onvelocity used */
extern long ticks, endticks, h_ticks, lasttick;

/* beamstyles from .midtomup file */
typedef struct {
  int bnum;
  int bdnom;
  char style[40];
} Bstyle;

extern Bstyle bstyles[50];
extern int bwhre;

extern int tied[MAXTRK];
/*
 * Keeps current measure of notes.  We'll use
 * this to avoid doing two accidentals on the
 * same note in the same measure.
 */

#define TABSIZE 32768

extern int accnotes[TABSIZE];
extern int accwhre;
/*
 * Similar to accnotes, but kept in seven note
 * format.  Instead of midi number, we keep note
 * name (as 1-7), octave, and accidental.
 */
extern int sccnotes[TABSIZE][3];
extern int sccwhre;
/*
 * Hold key signatures offsets.
 */
extern int key[MAXTRK];
extern int hkey[MAXTRK];
/*
 * tick and new key info
 */
typedef struct {
  long when;
  short nkey;
} Key;

#define MAXKEYSIG 100
extern Key midikey[MAXKEYSIG];
/* What is the current key? */
extern int mwhre;
/* Are we in a triplet? */
extern int triplet;
/* fudge to cut down number of onvelocity insertions */
extern int velfudge;
/* length of shortest note we can resolve */
extern int shortest;
extern FILE *out;

extern void mtm_error(char *msg);	// display an error message
extern int  mtm_getc(void);		// read a byte from the midi file
extern void mtm_trackstart(void);	// process start of track entry
extern void mtm_arbitrary(int, char *);	// arbitrary meta-event
extern void mtm_trackend(void);		// process end of track entry
extern void mtm_header(int format, int ntrks, int ldivision);
extern void mtm_seqnum(int seq);	// process sequence number
extern void mtm_text(int type, int leng, char *mess);
extern void mtm_eot(void);
extern void mtm_tempo(long ltempo);
extern void mtm_smpte(int hr, int mn, int se, int fr, int ff);
extern void mtm_timesig(int nn, int dd, int cc, int bb);
extern void mtm_keysig(int sf, int mi);
extern void mtm_seqspecific(int type, int leng, char *mess);
extern void mtm_metamisc(int type, int leng, char *mess);
extern void mtm_sysex(int leng, char *mess);
extern void mtm_noteoff(int chan, int pitch, int vol);
extern void mtm_noteon(int chan, int pitch, int vol);
extern void mtm_pressure(int chan, int pitch, int press);
extern void mtm_parameter(int chan, int control, int value);
extern void mtm_pitchbend(int chan, int msb, int lsb);
extern void mtm_program(int chan, int program);
extern void mtm_chanpressure(int chan, int press);
extern void mtm_metaspecial(int type, int leng, char *mess);

extern void poctave(int trck, int nval);
extern int goctave(int trck, int nval);

// following routines are in pnote.c
extern void pnote(int nval, int trk, int tied);
extern void resetaccs(int trk);
extern void setkeysig(int trk, int key);

#endif
