/*
 * Pnote.c
 *
 * Routine to print correct note and accidental given the
 * midi note value and track.
 *
 */
#include <stdio.h>
#include <stdlib.h>

#include "mtm.h"

// static int defoct = 4;

static char name[32][128];
static char acc[32][128];
static char accp[32][128];

static char *scales[] = {
/* C C#D D#E F F#G G#A A#B  */
  "cnd-dne-f-fng-gna-anb-c-",		/* 7& C& major */
  "cnd-dne-enf g-gna-anb-c-",		/* 6& G& major */
  "c d-dne-enf g-gna-anb-bn",		/* 5& D& major */
  "c d-dne-enf f#g a-anb-bn",		/* 4& A& major */
  "c c#d e-enf f#g a-anb-bn",		/* 3& E& major */
  "c c#d e-enf f#g a&a b-bn",		/* 2& B& Major */
  "c c#d e&e f f#g a&a b-bn",		/* 1& F  major */
  "c c#d e&e f g&g g#a b&b ",		/* 0& C  major */
  "c c#d e&e fnf+g a&a b&b ",		/* 1# G  major */
  "cnc+d e&e fnf+g a&a b&b ",		/* 2# D  major */
  "cnc+d e&e fnf+gng+a b&b ",		/* 3# A  major */
  "cnc+dnd+e fnf+gng+a b&b ",		/* 4# E  major */
  "cnc+dnd+e fnf+gng+ana+b ",		/* 5# B  major */
  "cnc+dnd+ene+f+gng+ana+b ",		/* 6# F# major */
  "b+c+dnd+ene+f+gng+ana+bn",		/* 7# C# major */
};

extern FILE *out;

void setkeysig(int trk, int key) {
  int i;
  char *p;
  
  key += 7;
  if (key < 0 || key > 14)
    {
      fprintf(stderr, "mtm: key outside range -7..+7\n");
      exit(1);
    }
  p = scales[key];
  for (i=0; i<128; i++)
    {
      if (p[0] == 0)
	p = scales[key];
      name[trk][i] = p[0];
      acc[trk][i] = p[1];
      accp[trk][i] = 0;
      p += 2;
    }
}

void resetaccs(int trk) {
  int i;
  for (i=0; i<128; i++)
    accp[trk][i] = ' ';
}

void pnote(int nval, int trk, int tied) {
  int o = goctave(trk, nval);
  int n = name[trk][nval];
  int a = acc[trk][nval];
  int z = 7 * o + (n-'a');

  if (accp[trk][z] == ' ') {
    if (a == '+' || a == '-')
      accp[trk][z] = ' ';
    else
      accp[trk][z] = a;
  }
  else if (accp[trk][z] == a)
    a = ' ';
  else if (a == ' ') {
    a = 'n';
    accp[trk][z] = a;
  }
  else if (a == '-') {
    a = '&';
    accp[trk][z] = ' ';
  }
  else if (a == '+') {
    a = '#';
    accp[trk][z] = ' ';
  }
  fputc(n, out);
  if (a == ' ' || tied != 0)
    return;
  if (a == '-' || a == '+')
    return;
  fputc(a, out);
}
